@@ -19,31 +19,31 @@
 
 #   Routines for toeplitz SSA
 
-tcirc.old <- function(F, L = (N - 1) %/% 2) {
-  N <- length(F);
+tcirc.old <- function(F, L = (N + 1) %/% 2) {
+  N <- length(F)
 
   f <- fft(c(F, rep(0, L-1)))
   R <- fft(f * Conj(f), inverse = TRUE)[1:L] / (N+L-1) / seq(from = N, to = N-L+1, by = -1)
 
-  .res <- list();
-  .res$C <- as.vector(fft(c(R, rev(R[-1]))));
-  .res$L <- L;
-  return (.res);
+  .res <- list()
+  .res$C <- as.vector(fft(c(R, rev(R[-1]))))
+  .res$L <- L
+  return (.res)
 }
 
 tmatmul.old <- function(C, v) {
-  v <- as.vector(fft(C$C * fft(c(v, rep(0, C$L-1))), inverse = TRUE));
-  Re((v/length(C$C))[1:C$L]);
+  v <- as.vector(fft(C$C * fft(c(v, rep(0, C$L-1))), inverse = TRUE))
+  Re((v/length(C$C))[1:C$L])
 }
 
 
 Lcor <- function(F, L) {
   storage.mode(F) <- "double"
-  storage.mode(L) <- "integer";
-  .Call("Lcor", F, L);
+  storage.mode(L) <- "integer"
+  .Call("Lcor", F, L)
 }
 
-new.tmat <- function(F, fft.plan = NULL, L = (N - 1) %/% 2) {
+new.tmat <- function(F, L = (N + 1) %/% 2, fft.plan = NULL) {
   N <- length(F)
   R <- Lcor(F, L)
 
@@ -64,166 +64,180 @@ is.tmat <- function(t) {
 }
 
 tmatmul <- function(tmat, v, transposed = FALSE) {
-  storage.mode(v) <- "double";
-  storage.mode(transposed) <- "logical";
-  .Call("tmatmul", tmat, v, transposed);
+  storage.mode(v) <- "double"
+  storage.mode(transposed) <- "logical"
+  .Call("tmatmul", tmat, v, transposed)
 }
 
 .init.toeplitz.ssa <- function(x, ...) {
   # Initialize FFT plan
-  .set(x, "fft.plan", fft.plan.1d(x$length))
+  .get.or.create.fft.plan(x)
 
   x
 }
 
 .hankelize.one.toeplitz.ssa <- .hankelize.one.1d.ssa
 
+.get.or.create.tmat <- function(x) {
+  .get.or.create(x, "tmat", new.tmat(F = x$F, L = x$window))
+}
+
 decompose.toeplitz.ssa.nutrlan <- function(x,
                                            neig = min(50, L, K),
                                            ...) {
-  N <- x$length; L <- x$window; K <- N - L + 1;
+  N <- x$length; L <- x$window; K <- N - L + 1
 
-  F <- .get(x, "F");
+  F <- .get(x, "F")
+  h <- .get.or.create.hmat(x)
 
-  h <- .get(x, "hmat", allow.null = TRUE);
-  if (is.null(h)) {
-    fft.plan <- .get(x, "fft.plan")
-    h <- new.hmat(F, fft.plan = fft.plan, L = L)
-  }
+  olambda <- .get(x, "olambda", allow.null = TRUE)
+  U <- .get(x, "U", allow.null = TRUE)
 
-  olambda <- .get(x, "olambda", allow.null = TRUE);
-  U <- .get(x, "U", allow.null = TRUE);
-
-  T <- .get(x, "tmat", allow.null = TRUE);
-  if (is.null(T)) {
-    T <- new.tmat(F, L = L);
-  }
+  T <- .get.or.create.tmat(x)
 
   S <- trlan.eigen(T, neig = neig, ...,
-                   lambda = olambda, U = U);
+                   lambda = olambda, U = U)
 
   # Save results
-  .set(x, "hmat", h);
-  .set(x, "tmat", T);
-  .set(x, "olambda", S$d);
+  .set(x, "olambda", S$d)
   if (!is.null(S$u))
-    .set(x, "U", S$u);
+    .set(x, "U", S$u)
 
-  num <- length(S$d);
-  lambda <- numeric(num);
-  V <- matrix(nrow = K, ncol = num);
+  num <- length(S$d)
+  lambda <- numeric(num)
+  V <- matrix(nrow = K, ncol = num)
   for (i in 1:num) {
-    Z <- hmatmul(h, S$u[, i], transposed = TRUE);
-    lambda[i] <- sqrt(sum(Z^2));
-    V[, i] <- Z / lambda[i];
+    Z <- hmatmul(h, S$u[, i], transposed = TRUE)
+    lambda[i] <- sqrt(sum(Z^2))
+    V[, i] <- Z / lambda[i]
   }
 
   # Save results
-  .set(x, "lambda", lambda);
-  .set(x, "V", V);
+  .set(x, "lambda", lambda)
+  .set(x, "V", V)
 
-  x;
+  x
 }
 
 decompose.toeplitz.ssa.eigen <- function(x, ...,
                                          force.continue = FALSE) {
-  N <- x$length; L <- x$window; K <- N - L + 1;
+  N <- x$length; L <- x$window; K <- N - L + 1
 
   # Check, whether continuation of decomposition is requested
   if (!force.continue && nlambda(x) > 0)
     stop("Continuation of decompostion is not supported for this method.")
 
   # Build hankel matrix
-  F <- .get(x, "F");
-  fft.plan <- .get(x, "fft.plan")
-  h <- new.hmat(F, fft.plan = fft.plan, L = L);
+  F <- .get(x, "F")
+  h <- .get.or.create.hmat(x)
 
   # Do decomposition
   if ("neig" %in% names(list(...)))
     warning("'neig' option ignored for SSA method 'eigen', computing EVERYTHING",
             immediate. = TRUE)
 
-  C <- toeplitz(Lcor(F, L));
-  S <- eigen(C, symmetric = TRUE);
+  C <- toeplitz(Lcor(F, L))
+  S <- eigen(C, symmetric = TRUE)
 
-  .set(x, "U", S$vectors);
+  .set(x, "U", S$vectors)
 
-  lambda <- numeric(L);
-  V <- matrix(nrow = K, ncol = L);
+  lambda <- numeric(L)
+  V <- matrix(nrow = K, ncol = L)
   for (i in 1:L) {
-    Z <- hmatmul(h, S$vectors[,i], transposed = TRUE);
-    lambda[i] <- sqrt(sum(Z^2));
-    V[, i] <- Z / lambda[i];
+    Z <- hmatmul(h, S$vectors[,i], transposed = TRUE)
+    lambda[i] <- sqrt(sum(Z^2))
+    V[, i] <- Z / lambda[i]
   }
 
   # Save results
-  .set(x, "lambda", lambda);
-  .set(x, "V", V);
+  .set(x, "lambda", lambda)
+  .set(x, "V", V)
 
-  x;
+  x
 }
 
-decompose.toeplitz.ssa.svd <- function(x, ...) {
-  stop("'SVD' method is not applicable to toeplitz SSA");
+decompose.toeplitz.ssa.svd <- function(x,
+                                       neig = min(L, K),
+                                       ...,
+                                       force.continue = FALSE) {
+  N <- x$length; L <- x$window; K <- N - L + 1
+
+  # Check, whether continuation of decomposition is requested
+  if (!force.continue && nlambda(x) > 0)
+    stop("Continuation of decompostion is not supported for this method.")
+
+  # Build hankel matrix
+  F <- .get(x, "F")
+  h <- .get.or.create.hmat(x)
+
+  # Do decomposition
+  C <- toeplitz(Lcor(F, L))
+  S <- svd(C, nu = neig, nv = neig)
+
+  .set(x, "U", S$u)
+
+  lambda <- numeric(neig)
+  V <- matrix(nrow = K, ncol = neig)
+  for (i in 1:neig) {
+    Z <- hmatmul(h, S$u[,i], transposed = TRUE)
+    lambda[i] <- sqrt(sum(Z^2))
+    V[, i] <- Z / lambda[i]
+  }
+
+  # Save results
+  .set(x, "lambda", lambda)
+  .set(x, "V", V)
+
+  x
 }
 
 decompose.toeplitz.ssa.propack <- function(x,
                                            neig = min(50, L, K),
                                            ...,
                                            force.continue = FALSE) {
-  N <- x$length; L <- x$window; K <- N - L + 1;
+  N <- x$length; L <- x$window; K <- N - L + 1
 
   # Check, whether continuation of decomposition is requested
   if (!force.continue && nlambda(x) > 0)
     stop("Continuation of decompostion is not yet implemented for this method.");
 
-  F <- .get(x, "F");
+  F <- .get(x, "F")
+  h <- .get.or.create.hmat(x)
 
-  h <- .get(x, "hmat", allow.null = TRUE);
-  if (is.null(h)) {
-    fft.plan <- .get(x, "fft.plan")
-    h <- new.hmat(F, fft.plan = fft.plan, L = L);
-  }
+  olambda <- .get(x, "olambda", allow.null = TRUE)
+  U <- .get(x, "U", allow.null = TRUE)
 
-  olambda <- .get(x, "olambda", allow.null = TRUE);
-  U <- .get(x, "U", allow.null = TRUE);
+  T <- .get.or.create.tmat(x)
 
-  T <- .get(x, "tmat", allow.null = TRUE);
-  if (is.null(T)) {
-    T <- new.tmat(F, L = L)
-  }
-
-  S <- propack.svd(T, neig = neig, ...);
+  S <- propack.svd(T, neig = neig, ...)
 
   # Save results
-  .set(x, "hmat", h);
-  .set(x, "tmat", T);
-  .set(x, "olambda", S$d);
+  .set(x, "olambda", S$d)
   if (!is.null(S$u))
-    .set(x, "U", S$u);
+    .set(x, "U", S$u)
 
-  num <- length(S$d);
-  lambda <- numeric(num);
-  V <- matrix(nrow = K, ncol = num);
+  num <- length(S$d)
+  lambda <- numeric(num)
+  V <- matrix(nrow = K, ncol = num)
   for (i in 1:num) {
-    Z <- hmatmul(h, S$u[, i], transposed = TRUE);
-    lambda[i] <- sqrt(sum(Z^2));
-    V[, i] <- Z / lambda[i];
+    Z <- hmatmul(h, S$u[, i], transposed = TRUE)
+    lambda[i] <- sqrt(sum(Z^2))
+    V[, i] <- Z / lambda[i]
   }
 
   # Save results
-  .set(x, "lambda", lambda);
-  .set(x, "V", V);
+  .set(x, "lambda", lambda)
+  .set(x, "V", V)
 
-  x;
+  x
 }
 
 decompose.toeplitz.ssa <- function(x,
                                    neig = min(50, L, K),
                                    ...,
                                    force.continue = FALSE) {
-  N <- x$length; L <- x$window; K <- N - L + 1;
-  stop("Unsupported SVD method for Toeplitz SSA!");
+  N <- x$length; L <- x$window; K <- N - L + 1
+  stop("Unsupported SVD method for Toeplitz SSA!")
 }
 
 calc.v.toeplitz.ssa <- function(x, idx, env = .GlobalEnv, ...)
